import os
import platform
import sys
import unittest

import packaging.version

try:
    import importlib.metadata as importlib_metadata  # get package version numbers - available in stdlib from python 3.8
except ImportError:
    # noinspection PyUnresolvedReferences
    import importlib_metadata

import pyoslog
import pyoslog_test_globals
from pyoslog import core as pyoslog_core

print('Testing pyoslog', packaging.version.Version(importlib_metadata.version('pyoslog')), 'setup')


class TestSetup(unittest.TestCase):
    def test_constants(self):
        self.assertIsInstance(pyoslog.OS_LOG_DEFAULT, pyoslog_core.os_log_t)
        self.assertEqual(str(pyoslog.OS_LOG_DEFAULT), '<os_log_t (OS_LOG_DEFAULT)>')

        self.assertIsInstance(pyoslog.OS_LOG_DISABLED, pyoslog_core.os_log_t)
        self.assertEqual(str(pyoslog.OS_LOG_DISABLED), '<os_log_t (OS_LOG_DISABLED)>')

        self.assertEqual(pyoslog.OS_LOG_TYPE_DEFAULT, pyoslog_test_globals.TestLogTypes.OS_LOG_TYPE_DEFAULT.value)
        self.assertEqual(pyoslog.OS_LOG_TYPE_INFO, pyoslog_test_globals.TestLogTypes.OS_LOG_TYPE_INFO.value)
        self.assertEqual(pyoslog.OS_LOG_TYPE_DEBUG, pyoslog_test_globals.TestLogTypes.OS_LOG_TYPE_DEBUG.value)
        self.assertEqual(pyoslog.OS_LOG_TYPE_ERROR, pyoslog_test_globals.TestLogTypes.OS_LOG_TYPE_ERROR.value)
        self.assertEqual(pyoslog.OS_LOG_TYPE_FAULT, pyoslog_test_globals.TestLogTypes.OS_LOG_TYPE_FAULT.value)

    def test_is_supported(self):
        # a little pointless since our platform check and that from is_supported() are identical, but no other option
        matching_platform = sys.platform == 'darwin' and sys.version_info >= (3, 0,) and float(
            '.'.join(platform.mac_ver()[0].split('.')[:2])) >= 10.12
        self.assertEqual(pyoslog.is_supported(), matching_platform)

        # a little contrived, but might as well test the documentation building special case
        os.environ['PYOSLOG_OVERRIDE_IS_SUPPORTED'] = '1'
        self.assertEqual(pyoslog.is_supported(), True)
        del os.environ['PYOSLOG_OVERRIDE_IS_SUPPORTED']

    def test_os_log_create(self):
        log = pyoslog.os_log_create(pyoslog_test_globals.LOG_SUBSYSTEM, pyoslog_test_globals.LOG_CATEGORY)
        self.assertIsInstance(log, pyoslog_core.os_log_t)
        self.assertEqual(str(log), '<os_log_t (%s:%s)>' % (pyoslog_test_globals.LOG_SUBSYSTEM,
                                                           pyoslog_test_globals.LOG_CATEGORY))

        # PyArg_ParseTuple in _pyoslog.c handles type validation - just ensure a string is required and test boundaries
        self.assertRaises(TypeError, pyoslog.os_log_create, (None, None))
        self.assertRaises(TypeError, pyoslog.os_log_create, (pyoslog_test_globals.LOG_SUBSYSTEM, None))
        self.assertRaises(TypeError, pyoslog.os_log_create, (None, pyoslog_test_globals.LOG_CATEGORY))

        self.assertRaises(TypeError, pyoslog.os_log_create, ('', ''))
        self.assertRaises(TypeError, pyoslog.os_log_create, (pyoslog_test_globals.LOG_SUBSYSTEM, ''))
        self.assertRaises(TypeError, pyoslog.os_log_create, ('', pyoslog_test_globals.LOG_CATEGORY))

        self.assertRaises(TypeError, pyoslog.os_log_create, (250 * 'p', 255 * 'p'))
        self.assertRaises(TypeError, pyoslog.os_log_create, (pyoslog_test_globals.LOG_SUBSYSTEM, 255 * 'p'))
        self.assertRaises(TypeError, pyoslog.os_log_create, (250 * 'p', pyoslog_test_globals.LOG_CATEGORY))


if __name__ == '__main__':
    unittest.main()
